/**
 * @ngdoc overview
 * @name Project BonusKing
 */
var app = angular.module('BonusKing', [
  'ngRoute',
  'ngSanitize',
  'ngTouch',

  'ui.router',
  'ionic',
  'ngCordova',
  'ngSails',
  'ImgCache',
  'pascalprecht.translate',
  'ui.bootstrap',

  'PWT.Config',

  'PWT.Resources',
  'PWT.Auth',

  'BonusKing.StateConstants',
  'BonusKing.Utils',
  'BonusKing.GoogleAnalytics',
  'BonusKing.ClientStateReporter',
  'BonusKing.GameProtocol',
  'BonusKing.PushNotification',
  'BonusKing.CoinAnimation',
  'BonusKing.ImagePreloader',
  'BonusKing.Audio',
  'BonusKing.Navigation',

  'BonusKing.views.lobby',
  'BonusKing.views.game',
  'BonusKing.views.vouchers',
  'BonusKing.views.voucherDetails',
  'BonusKing.views.bonus',
  'BonusKing.views.interstitial',
  'BonusKing.views.launchScreen',
  'BonusKing.views.bonusWheel',
  'BonusKing.views.dailyBonus',
  'BonusKing.views.giftBox',
  'BonusKing.views.errorMessage',
  'BonusKing.views.countrySelection',
  'BonusKing.views.settings',

  'BonusKing.views.styleGuide',

  'BonusKing.modals.shop',
  'BonusKing.modals.xp',
  'BonusKing.modals.levelUp',
  'BonusKing.modals.promoCode',
  'BonusKing.modals.facebookConnect',
  'BonusKing.views.bye',

  ])

.config([
  '$translateProvider',
  '$httpProvider',
  '$logProvider',
  '$stateProvider',
  '$urlRouterProvider',
  '$sailsProvider',
  'ImgCacheProvider',
  'Options',

  function($translateProvider,
    $httpProvider,
    $logProvider,
    $stateProvider,
    $urlRouterProvider,
    $sailsProvider,
    ImgCacheProvider,
    Options) {

    // -----------------------------------
    // Debug stuff
    // -----------------------------------
    $logProvider.debugEnabled(Options.debug);

    // -----------------------------------
    // Sails.js sockets
    // -----------------------------------
    $sailsProvider.url = Options.api.url;
    $sailsProvider.config.transports = ['websocket'];
    $sailsProvider.debug = Options.debug;
    $sailsProvider.interceptors.push('AuthInterceptor');
    $sailsProvider.interceptors.push('HttpInterceptor');

    // -----------------------------------
    // Html5 mode and cors stuff.
    // -----------------------------------
    $httpProvider.defaults.useXDomain = true;
    delete $httpProvider.defaults.headers.common['X-Requested-With'];

    // This is needed to send cookies along xdomain requests.
    $httpProvider.defaults.withCredentials = true;

    // -----------------------------------
    // Http interceptors for authentication
    // -----------------------------------
    $httpProvider.interceptors.push('AuthInterceptor');
    $httpProvider.interceptors.push('HttpInterceptor');

    // -----------------------------------
    // Internationalisation
    // -----------------------------------

    $translateProvider.preferredLanguage('de');
    $translateProvider.fallbackLanguage('en');
    $translateProvider.useSanitizeValueStrategy('escaped');
    $translateProvider
      .useStaticFilesLoader({
        prefix: 'lang/',
        suffix: '.json',
      });

    // -----------------------------------
    // Image caching
    // -----------------------------------

    ImgCacheProvider.manualInit = true;
    ImgCacheProvider.setOption('debug', true);
    ImgCacheProvider.setOption('usePersistentCache', true);
    ImgCacheProvider.setOption('localCacheFolder', 'BonusKing');
  },
])

.run(function(
  $location,
  $window,
  $log,
  $document,
  $rootScope,
  $ionicPlatform,
  $state,
  $translate,
  ClientStateReporter,
  Auth,
  Interstitial,
  Game,
  SHApplication,
  ImgCache,
  $cordovaStatusbar,
  $localstorage,
  Navigation,
  SHAudio,
  GA) {

  'use strict';

  // Needed on the mobile when opening the app.
  $ionicPlatform.on('resume', applicationDidFinishLaunching);

  // Inoinc platform loaded and ready
  $ionicPlatform.ready(applicationDidFinishLaunching);

  // Harware back button
  // $ionicPlatform.onHardwareBackButton(hardwareButtonPressed);
  $ionicPlatform.registerBackButtonAction(hardwareButtonPressed, 100);

  /**
   * A function called when the application did finish launcing and ionic
   * is initialised and ready
   */
  function applicationDidFinishLaunching() {
    $log.info('applicationDidFinishLaunching');

    /**
     * Logic for mobile client only
     */
    if (window.cordova) {

      /**
       * Locking orientation to landscape
       */
      screen.lockOrientation('landscape');

      /**
       * KeepAwake. Insomnia plugin.
       */
      if (window.plugins.insomnia) {
        window.plugins.insomnia.keepAwake();
      }

      /**
       * Hiding the status bar.
       */
      $cordovaStatusbar.hide();

      /**
       * Hide the accessory bar by default (remove this to show the accessory
       * bar above the keyboard for form inputs)
       */
      if (window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
      }

      if (navigator.globalization) {
        navigator
          .globalization
            .getPreferredLanguage(function(language) {

              $translate
                .use((language.value)
                .split('-')[0])
                .then(function(data) {
                  $log.debug('SH APP', 'Language is now: ' + data);
                  $localstorage.set('language', data);
                }, function(e) {
                  $log.warn('SH APP', 'failed fethcing language' + e);
                });
            }, null);
      }
    }

    // Initiating image cache
    ImgCache.$init();

    // Setting default language
    // TODO: this is only for the browser. overwritten wiht ionic
    $localstorage.set('language', 'en');

    var isFirstStart = ($localstorage.get('isFirstStart') != 'no');

    // Always show launch screen
    $state.go('launchScreen');

    if (isFirstStart) {
      $localstorage.set('isFirstStart', 'no');
      Navigation.firstStart();
    } else {
      Navigation.authenticate();
    }

    /**
     * Adding event listener on every click and playing click sound
     */
    // SHAudio.preloadSimple('click');
    // $document
    //   .on('click', function(event) {
    //     SHAudio.play('click');
    //   })

    /**
     * Adding listener on app exit and report to server
     */
    document.addEventListener('pause', function() {

      // Stopping all sounds on exit.
      SHAudio.stopAll();

      // Sending notification for user leaving the app
      var appStateObj = {
        key: SHApplication.didExit,
        value: Auth.getCurrentUser().id,
      };
      ClientStateReporter.sendAppStateChange(appStateObj);

    }, false);
  }

  function hardwareButtonPressed(event) {
    $log.debug('SHHardwareButtonPressed');
    $rootScope.$broadcast(SHApplication.hardwareButtonPressed);
    event.preventDefault();
    event.stopPropagation();
  }
})

.factory('HttpInterceptor', ['$log', '$q', function($log, $q) {
  'use strict';

  return {

    request: function(config) {
      var firstPathComponent = config.url.split('/')[0];
      var isLocalRequest = (firstPathComponent == 'views' ||
        firstPathComponent == 'template');

      if (!isLocalRequest) {
        $log.debug('BonusKingAdmin #',
        '[http request] ' + config.method + ': ' + config.url, config);
      }

      return config;
    },

    response: function(response) {
      var isLocalRequest = (response.config.url.split('/')[0] == 'views');
      if (!isLocalRequest) {
        $log.debug('BonusKing #',
          '[http response]',
          response.config.url,
          response.data);
      }
      return response;
    },

    responseError: function(rejection) {
      $log.warn('BonusKing #', '[http response error]', rejection);
      return $q.reject(rejection);
    },
  };
},]);